require 'spec_helper'

describe StatisticalMethods::Core::Array do
  let(:delta) { 0.001 }

  describe '#addition' do
    it { expect([].addition).to eq(nil) }
    it { expect([0].addition).to be_within(delta).of(0) }
    it { expect([0, 1].addition).to be_within(delta).of(1) }
    it { expect([0, 1, 2].addition).to be_within(delta).of(3) }
    it { expect([0, 1, 2, 3].addition).to be_within(delta).of(6) }
    it { expect([0, 1, 2, 3, 4].addition).to be_within(delta).of(10) }
  end

  describe '#multiplication' do
    it { expect([].multiplication).to eq(nil) }
    it { expect([0].multiplication).to be_within(delta).of(0) }
    it { expect([1].multiplication).to be_within(delta).of(1) }
    it { expect([1, 2].multiplication).to be_within(delta).of(2) }
    it { expect([1, 2, 3].multiplication).to be_within(delta).of(6) }
    it { expect([1, 2, 3, 4].multiplication).to be_within(delta).of(24) }
  end

  describe '#power' do
    it { expect([0].power(1)).to eq([0]) }
    it { expect([0, 1].power(1)).to eq([0, 1]) }
    it { expect([0, 1, 2].power(2)).to eq([0, 1, 4]) }
    it { expect([0, 1, 2, 3].power(3)).to eq([0, 1, 8, 27]) }
    it { expect([0, 1, 2, 3, 4].power(4)).to eq([0, 1, 16, 81, 256]) }
  end

  describe '#harmonic' do
    it { expect([].harmonic).to eq([]) }
    it { expect([1].harmonic).to eq([1]) }
    it { expect([1, 2].harmonic).to eq([1, 0.5]) }
    it { expect([1, 2, 3].harmonic).to eq([1, 0.5, 1.0 / 3.0]) }
    it { expect([1, 2, 3, 4].harmonic).to eq([1, 0.5, 1.0 / 3.0, 0.25]) }
  end

  describe '#group_by_count' do
    it { expect([].group_by_count).to eq({}) }
    it { expect([0].group_by_count).to eq(0 => 1) }
    it { expect([0, 1].group_by_count).to eq(0 => 1, 1 => 1) }
    it { expect([0, 1, 2].group_by_count).to eq(0 => 1, 1 => 1, 2 => 1) }
    it { expect([0, 1, 1, 1].group_by_count).to eq(0 => 1, 1 => 3) }
  end
end
