require 'spec_helper'

describe StatisticalMethods::SummaryStatistic::Location::Average do
  context '#arithmetic_mean' do
    shared_examples_for :calculation do
      let(:delta) { 0.001 }
      subject { described_class.new(array) }
      it { expect(subject.arithmetic_mean).to be_within(delta).of(result) }
    end

    it do
      subject = described_class.new([])
      expect(subject.arithmetic_mean).to be_nil
    end

    it_behaves_like :calculation do
      let(:array) { [0] }
      let(:result) { 0 }
    end

    it_behaves_like :calculation do
      let(:array) { [0, 1] }
      let(:result) { 0.5 }
    end

    it_behaves_like :calculation do
      let(:array) { [0, 0, 1] }
      let(:result) { 0.3333 }
    end
  end

  context '#geometric_mean' do
    shared_examples_for :calculation do
      let(:delta) { 0.001 }
      subject { described_class.new(array) }
      specify { expect(subject.geometric_mean).to be_within(delta).of(result) }
    end

    it do
      subject = described_class.new([])
      expect(subject.geometric_mean).to be_nil
    end

    it_behaves_like :calculation do
      let(:array) { [0] }
      let(:result) { 0 }
    end

    it_behaves_like :calculation do
      let(:array) { [1] }
      let(:result) { 1 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:result) { 4 }
    end

    it_behaves_like :calculation do
      let(:array) { [4, 1, 0.03125].shuffle }
      let(:result) { 0.5 }
    end
  end

  context '#harmonic_mean' do
    shared_examples_for :calculation do
      let(:delta) { 0.001 }
      subject { described_class.new(array) }
      it { expect(subject.harmonic_mean).to be_within(delta).of(result) }
    end

    it do
      subject = described_class.new([])
      expect(subject.harmonic_mean).to be_nil
    end

    it do
      subject = described_class.new([0])
      expect(subject.harmonic_mean).to be_nil
    end

    it_behaves_like :calculation do
      let(:array) { [1] }
      let(:result) { 1 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 2] }
      let(:result) { 2 }
    end

    it_behaves_like :calculation do
      let(:array) { [1, 2, 4].shuffle }
      let(:result) { 12.0 / 7.0 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 2, 5, 7].shuffle }
      let(:result) { 140.0 / 47.0 }
    end
  end

  context '#quadratic_mean' do
    shared_examples_for :calculation do
      let(:delta) { 0.001 }
      subject { described_class.new(array) }
      it { expect(subject.quadratic_mean).to be_within(delta).of(result) }
    end

    it do
      subject = described_class.new([])
      expect(subject.quadratic_mean).to be_nil
    end

    it_behaves_like :calculation do
      let(:array) { [0] }
      let(:result) { 0 }
    end

    it_behaves_like :calculation do
      let(:array) { [1] }
      let(:result) { 1 }
    end

    it_behaves_like :calculation do
      let(:array) { [1, 1] }
      let(:result) { 1 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 2] }
      let(:result) { 2 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 2, 5, 7].shuffle }
      let(:delta) { 0.01 }
      let(:result) { 4.53 }
    end
  end

  context '#power_mean' do
    shared_examples_for :calculation do
      let(:delta) { 0.001 }
      let(:exponent) { 0 }
      subject { described_class.new(array) }
      it do
        expect(subject.power_mean exponent).to be_within(delta).of(result)
      end
    end

    it do
      subject = described_class.new([])
      expect(subject.power_mean 0).to be_nil
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:exponent) { 0 }
      let(:result) { 4 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:exponent) { 1 }
      let(:result) { 5 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:exponent) { 2 }
      let(:delta) { 0.01 }
      let(:result) { 5.83 }
    end

    it_behaves_like :calculation do
      let(:array) { [1, 2, 3, 4, 5] }
      let(:exponent) { 3 }
      let(:delta) { 0.01 }
      let(:result) { 3.56 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:exponent) { -Float::INFINITY }
      let(:result) { 2 }
    end

    it_behaves_like :calculation do
      let(:array) { [2, 8] }
      let(:exponent) { +Float::INFINITY }
      let(:result) { 8 }
    end
  end
end
